use crate::ems::{Consumer, Producer, Service};
use crate::internal::utils;
use log::info;
use std::sync::Arc;
use tokio::join;
use tokio::sync::Mutex;

pub struct Pcs {
    pub id: String,
    pub producer: Producer,
    pub consumer: Arc<Mutex<Consumer>>,
}

impl Pcs {
    /// 初始化PCS
    pub async fn new(producer: Producer, consumer: Consumer) -> Self {
        let id = utils::generate_random_str(12);
        utils::log::init_log("inpower_iot_mgc_rs::ems::pcs::*", "logs/pcs.log").await;
        info!("PCS [{}] 初始化成功", id);
        Pcs {
            id,
            producer,
            consumer: Arc::new(Mutex::new(consumer)),
        }
    }
}

async fn read_task(producer: Producer) {
    loop {
        producer.send("PCS nihao".to_string()).await.unwrap();
        tokio::time::sleep(tokio::time::Duration::from_secs(5)).await;
    }
}

async fn write_task(consumer: Arc<Mutex<Consumer>>) {
    loop {
        let string = consumer.lock().await.recv().await.unwrap();
        info!("[PCS] {}", string);
    }
}

#[async_trait::async_trait]
impl Service for Pcs {
    /// PCS 服务
    /// 内部具有并发读写寄存器
    async fn serve(&self) {
        let read_handle = tokio::spawn(read_task(self.producer.clone()));
        let write_handle = tokio::spawn(write_task(self.consumer.clone()));
        let _ = join!(read_handle, write_handle);
    }
}
